@@ -64,11 +64,11 @@
 /* This module provides the platform specific entry and fail processing. The _plat__RunCommand()
    function is used to call to ExecuteCommand() in the TPM code. This function does whatever
    processing is necessary to set up the platform in anticipation of the call to the TPM including
-   setup for error processing. */
+   settup for error processing. */
 /* The _plat__Fail() function is called when there is a failure in the TPM. The TPM code will have
    set the flag to indicate that the TPM is in failure mode. This call will then recursively call
    ExecuteCommand() in order to build the failure mode response. When ExecuteCommand() returns to
-   _plat__Fail(), the platform will do some platform specif operation to return to the environment
+   _plat__Fail(), the platform will do some platform specific operation to return to the environment
    in which the TPM is executing. For a simulator, setjmp/longjmp is used. For an OS, a system exit
    to the OS would be appropriate. */
 /* C.11.2. Includes and locals */
