@@ -354,7 +354,7 @@ TPM_RC DoEVPGetIV(
 {
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
     OSSL_PARAM params[] = {
-        OSSL_PARAM_octet_ptr(OSSL_CIPHER_PARAM_UPDATED_IV, &iv, iv_len),
+        OSSL_PARAM_octet_string(OSSL_CIPHER_PARAM_UPDATED_IV, iv, iv_len),
         OSSL_PARAM_END
     };
     if (EVP_CIPHER_CTX_get_params(ctx, params) != 1)
@@ -1131,7 +1131,7 @@ UINT16 OSSLCryptKDFe(TPM_ALG_ID   hashAlg,  // IN: hash algorithm used in HMAC
     size_t      buffer_size = 0;
     UINT16      generated = 0;
     size_t      offset = 0;
-    EVP_KDF_CTX *ctx;
+    EVP_KDF_CTX *ctx = NULL;
     EVP_KDF     *kdf;
     char        *buffer;
     INT16       bytes;  // number of bytes to generate
@@ -1197,6 +1197,7 @@ UINT16 OSSLCryptKDFe(TPM_ALG_ID   hashAlg,  // IN: hash algorithm used in HMAC
     generated = bytes;
 
 out:
+    EVP_KDF_CTX_free(ctx);
     EVP_KDF_free(kdf);
     free(buffer);
 
