@@ -3,7 +3,6 @@
 /*		Splice the OpenSSL() library into the TPM code.    		*/
 /*			     Written by Ken Goldman				*/
 /*		       IBM Thomas J. Watson Research Center			*/
-/*            $Id: TpmToOsslSym.h 1619 2020-05-19 16:51:47Z kgoldman $		*/
 /*										*/
 /*  Licenses and Notices							*/
 /*										*/
@@ -55,143 +54,150 @@
 /*    arising in any way out of use or reliance upon this specification or any 	*/
 /*    information herein.							*/
 /*										*/
-/*  (c) Copyright IBM Corp. and others, 2016 - 2020				*/
+/*  (c) Copyright IBM Corp. and others, 2016 - 2023				*/
 /*										*/
 /********************************************************************************/
 
-/* B.2.3.2. TpmToOsslSym.h */
-/* B.2.3.2.1. Introduction */
-/* This header file is used to splice the OpenSSL() library into the TPM code. */
-/* The support required of a library are a hash module, a block cipher module and portions of a big
-   number library.  All of the library-dependent headers should have the same guard to that only the
-   first one gets defined. */
+//** Introduction
+//
+// This header file is used to 'splice' the OpenSSL library into the TPM code.
+//
+// The support required of a library are a hash module, a block cipher module and
+// portions of a big number library.
 
+// All of the library-dependent headers should have the same guard to that only the
+// first one gets defined.
 #ifndef SYM_LIB_DEFINED
 #define SYM_LIB_DEFINED
+
 #define SYM_LIB_OSSL
+
 #include <openssl/aes.h>
 #if ALG_TDES
 #include <openssl/des.h>
 #endif
 
 #if ALG_SM4
-#   if defined(OPENSSL_NO_SM4) || OPENSSL_VERSION_NUMBER < 0x10101010L
-#       undef ALG_SM4
-#       define ALG_SM4  ALG_NO
-#   elif OPENSSL_VERSION_NUMBER >= 0x10200000L
-#       include <openssl/sm4.h>
+#  if defined(OPENSSL_NO_SM4) || OPENSSL_VERSION_NUMBER < 0x10101010L
+#    error "Current version of OpenSSL doesn't support SM4"
+#   //elif OPENSSL_VERSION_NUMBER >= 0x10200000L		// libtpms deactivated
+#   //    include <openssl/sm4.h>				// libtpms deactivated
 #   else
-// OpenSSL 1.1.1 keeps smX.h headers in the include/crypto directory,
-// and they do not get installed as part of the libssl package
+#       include <openssl/evp.h>					// libtpms changed begin
+        typedef EVP_CIPHER_CTX* SM4_KEY;
+#       define SM4_ENCRYPT 1
+#       define SM4_DECRYPT 0
 
-#       define SM4_KEY_SCHEDULE  32
-
-typedef struct SM4_KEY_st {
-    uint32_t rk[SM4_KEY_SCHEDULE];
-} SM4_KEY;
-
-int SM4_set_key(const uint8_t *key, SM4_KEY *ks);
-void SM4_encrypt(const uint8_t *in, uint8_t *out, const SM4_KEY *ks);
-void SM4_decrypt(const uint8_t *in, uint8_t *out, const SM4_KEY *ks);
-#   endif // OpenSSL < 1.2
-#endif // ALG_SM4
+int SM4_set_encrypt_key(const uint8_t *key, SM4_KEY *ks);
+int SM4_set_decrypt_key(const uint8_t *key, SM4_KEY *ks);	// libtpms changed end
+void SM4_encrypt(const uint8_t* in, uint8_t* out, const SM4_KEY* ks);
+void SM4_decrypt(const uint8_t* in, uint8_t* out, const SM4_KEY* ks);
+void SM4_final(const SM4_KEY *ks);				// libtpms added
+#  endif  // OpenSSL < 1.2
+#endif    // ALG_SM4
 
 #if ALG_CAMELLIA
-#include <openssl/camellia.h>
+#  include <openssl/camellia.h>
 #endif
+
 #include <openssl/bn.h>
 #include <openssl/ossl_typ.h>
 
-/* B.2.2.3.2.	Links to the OpenSSL symmetric algorithms */
-// The Crypt functions that call the block encryption function use the parameters in the order:
-// a)	keySchedule
-// b)	in buffer
-// c) out buffer Since open SSL uses the order in encryptoCall_t above, need to swizzle the values
-// to the order required by the library.
+//***************************************************************
+//** Links to the OpenSSL symmetric algorithms.
+//***************************************************************
 
-#define SWIZZLE(keySchedule, in, out)					\
-    (const BYTE *)(in), (BYTE *)(out), (void *)(keySchedule)
+// The Crypt functions that call the block encryption function use the parameters
+// in the order:
+//  1) keySchedule
+//  2) in buffer
+//  3) out buffer
+// Since open SSL uses the order in encryptoCall_t above, need to swizzle the
+// values to the order required by the library.
+#define SWIZZLE(keySchedule, in, out)				\
+    (const BYTE*)(in), (BYTE*)(out), (void*)(keySchedule)
 
-// Define the order of parameters to the library functions that do block encryption and decryption.
-
-typedef void(*TpmCryptSetSymKeyCall_t)(
-				       const BYTE  *in,
-				       BYTE        *out,
-				       void *keySchedule
-				       );
+// Define the order of parameters to the library functions that do block encryption
+// and decryption.
+typedef void (*TpmCryptSetSymKeyCall_t)(const BYTE* in, BYTE* out, void* keySchedule);
 
+typedef void(*TpmCryptSymFinal_t)(void *keySchedule); /* libtpms added */
 #define SYM_ALIGNMENT   4 /* libtpms: keep old value */
 
-/* B.2.2.3.3.	Links to the OpenSSL AES code */
-/* Macros to set up the encryption/decryption key schedules */
-
+//***************************************************************
+//** Links to the OpenSSL AES code
+//***************************************************************
+// Macros to set up the encryption/decryption key schedules
+//
+// AES:
 #define TpmCryptSetEncryptKeyAES(key, keySizeInBits, schedule)		\
-    AES_set_encrypt_key((key), (keySizeInBits), (tpmKeyScheduleAES *)(schedule))
+    AES_set_encrypt_key((key), (keySizeInBits), (tpmKeyScheduleAES*)(schedule))
 #define TpmCryptSetDecryptKeyAES(key, keySizeInBits, schedule)		\
-    AES_set_decrypt_key((key), (keySizeInBits), (tpmKeyScheduleAES *)(schedule))
+    AES_set_decrypt_key((key), (keySizeInBits), (tpmKeyScheduleAES*)(schedule))
 
-/* Macros to alias encryption calls to specific algorithms. This should be used
-   sparingly. Currently, only used by CryptSym.c and CryptRand.c */
-/* When using these calls, to call the AES block encryption code, the caller should use:
-   TpmCryptEncryptAES(SWIZZLE(keySchedule, in, out)); */
-
-#define TpmCryptEncryptAES          AES_encrypt
-#define TpmCryptDecryptAES          AES_decrypt
-#define tpmKeyScheduleAES           AES_KEY
-
-/* B.2.2.3.4.	Links to the OpenSSL DES code */
-
-#if ALG_TDES && 0 // libtpms changed
-#include "TpmToOsslDesSupport_fp.h"
-#endif
+// Macros to alias encryption calls to specific algorithms. This should be used
+// sparingly. Currently, only used by CryptSym.c and CryptRand.c
+//
+// When using these calls, to call the AES block encryption code, the caller
+// should use:
+//      TpmCryptEncryptAES(SWIZZLE(keySchedule, in, out));
+#define TpmCryptEncryptAES AES_encrypt
+#define TpmCryptDecryptAES AES_decrypt
+#define tpmKeyScheduleAES  AES_KEY
+#define TpmCryptFinalAES   NULL  // libtpms added
 
 #define TpmCryptSetEncryptKeyTDES(key, keySizeInBits, schedule)		\
     TDES_set_encrypt_key((key), (keySizeInBits), (tpmKeyScheduleTDES *)(schedule))
 #define TpmCryptSetDecryptKeyTDES(key, keySizeInBits, schedule)		\
     TDES_set_encrypt_key((key), (keySizeInBits), (tpmKeyScheduleTDES *)(schedule))
 
-/* Macros to alias encryption calls to specific algorithms. This should be used
-   sparingly. Currently, only used by CryptRand.c */
-
 #define TpmCryptEncryptTDES         TDES_encrypt
 #define TpmCryptDecryptTDES         TDES_decrypt
 #define tpmKeyScheduleTDES          DES_key_schedule
+#define TpmCryptFinalTDES           NULL  // libtpms added
 
 #if ALG_TDES  // libtpms added begin
 #include "TpmToOsslDesSupport_fp.h"
 #endif        // libtpms added end
 
-/* B.2.2.3.5.	Links to the OpenSSL SM4 code */
-/* Macros to set up the encryption/decryption key schedules */
-
+//***************************************************************
+//** Links to the OpenSSL SM4 code
+//***************************************************************
+// Macros to set up the encryption/decryption key schedules
 #define TpmCryptSetEncryptKeySM4(key, keySizeInBits, schedule)	\
-    SM4_set_key((key), (tpmKeyScheduleSM4 *)(schedule))
+    SM4_set_encrypt_key((key), (tpmKeyScheduleSM4 *)(schedule)) /* libtpms changed */
 #define TpmCryptSetDecryptKeySM4(key, keySizeInBits, schedule)	\
-    SM4_set_key((key), (tpmKeyScheduleSM4 *)(schedule))
-/* Macros to alias encryption calls to specific algorithms. This should be used sparingly. */
+    SM4_set_decrypt_key((key), (tpmKeyScheduleSM4 *)(schedule)) /* libtpms changed */
 
-#define TpmCryptEncryptSM4          SM4_encrypt
-#define TpmCryptDecryptSM4          SM4_decrypt
-#define tpmKeyScheduleSM4           SM4_KEY
-
-/* B.2.2.3.6.	Links to the OpenSSL CAMELLIA code */
-/* Macros to set up the encryption/decryption key schedules */
+// Macros to alias encryption calls to specific algorithms. This should be used
+// sparingly.
+#define TpmCryptEncryptSM4 SM4_encrypt
+#define TpmCryptDecryptSM4 SM4_decrypt
+#define tpmKeyScheduleSM4  SM4_KEY
+#define TpmCryptFinalSM4   SM4_final // libtpms added
 
+//***************************************************************
+//** Links to the OpenSSL CAMELLIA code
+//***************************************************************
+// Macros to set up the encryption/decryption key schedules
 #define TpmCryptSetEncryptKeyCAMELLIA(key, keySizeInBits, schedule)	\
-    Camellia_set_key((key), (keySizeInBits), (tpmKeyScheduleCAMELLIA *)(schedule))
+    Camellia_set_key((key), (keySizeInBits), (tpmKeyScheduleCAMELLIA*)(schedule))
 #define TpmCryptSetDecryptKeyCAMELLIA(key, keySizeInBits, schedule)	\
-    Camellia_set_key((key), (keySizeInBits), (tpmKeyScheduleCAMELLIA *)(schedule))
+    Camellia_set_key((key), (keySizeInBits), (tpmKeyScheduleCAMELLIA*)(schedule))
 
-/* Macros to alias encryption calls to specific algorithms. This should be used sparingly. */
+// Macros to alias encryption calls to specific algorithms. This should be used
+// sparingly.
+#define TpmCryptEncryptCAMELLIA Camellia_encrypt
+#define TpmCryptDecryptCAMELLIA Camellia_decrypt
+#define tpmKeyScheduleCAMELLIA  CAMELLIA_KEY
+#define TpmCryptFinalCAMELLIA            NULL // libtpms added
 
-#define TpmCryptEncryptCAMELLIA          Camellia_encrypt
-#define TpmCryptDecryptCAMELLIA          Camellia_decrypt
-#define tpmKeyScheduleCAMELLIA           CAMELLIA_KEY
-
-/* Forward reference */
+// Forward reference
 
 // kgold typedef union tpmCryptKeySchedule_t tpmCryptKeySchedule_t;
 
-/* This definition would change if there were something to report */
+// This definition would change if there were something to report
 #define SymLibSimulationEnd()
-#endif // SYM_LIB_DEFINED
+
+#endif  // SYM_LIB_DEFINED
+
