@@ -39,6 +39,7 @@
 
 #include <config.h>
 
+#define _GNU_SOURCE
 #include <assert.h>
 #include <string.h>
 #if defined __FreeBSD__
@@ -267,6 +268,16 @@ TPM_RESULT TPMLIB_ValidateState(enum TPMLIB_StateType st,
     return tpm_iface[tpmvers_choice]->ValidateState(st, flags);
 }
 
+TPM_RESULT TPMLIB_SetProfile(const char *profile)
+{
+    return tpm_iface[tpmvers_choice]->SetProfile(profile);
+}
+
+TPM_BOOL TPMLIB_WasManufactured(void)
+{
+    return tpm_iface[tpmvers_choice]->WasManufactured();
+}
+
 static struct libtpms_callbacks libtpms_cbs;
 
 struct libtpms_callbacks *TPMLIB_GetCallbacks(void)
@@ -582,6 +593,18 @@ void TPMLIB_LogArray(unsigned int indent, const unsigned char *data,
     }
 }
 
+int TPMLIB_asprintf(char **strp, const char *fmt, ...)
+{
+    int ret;
+    va_list ap;
+
+    va_start(ap, fmt);
+    ret = vasprintf(strp, fmt, ap);
+    va_end(ap);
+
+    return ret;
+}
+
 void ClearCachedState(enum TPMLIB_StateType st)
 {
     free(cached_blobs[st].buffer);
@@ -636,6 +659,8 @@ TPM_RESULT CopyCachedState(enum TPMLIB_StateType st,
     *is_empty_buffer = (*buflen == BUFLEN_EMPTY_BUFFER);
 
     if (cached_blobs[st].buffer) {
+        assert(*buflen != BUFLEN_EMPTY_BUFFER);
+
         *buffer = malloc(*buflen);
         if (!*buffer) {
             TPMLIB_LogError("Could not allocate %u bytes.\n", *buflen);
